
/*
**    Ring Light
**
**   Copyright 2021 Guido Mersmann <geit@geit.de>
**   All rights reserved.
**
*/

/***************************************************************************/

#include "main.h"
#include "rl_animation.h"
#include "rl_display.h"
#include "rl_global.h"
#include "rl_ota.h"
#include "rl_render.h"
#include "rl_sound.h"
#include "rl_testmode.h"
#include "rl_time.h"
#include "rl_webserver.h"
#include "rl_wifi.h"

/***************************************************************************/

struct RingLightConfig rlc;

struct RingLightRuntimeData rlrd;

TaskHandle_t core0Loop;

/***************************************************************************/

static void CodeLoopCore0( void * pvParameters );

/***************************************************************************/

/* /// setup()
**
*/

/***************************************************************************/

void setup()
{
	debug(( "\nsetup()\n" ));

#if ENABLE_DEBUG
	Serial.begin( DEBUG_SERIALSPEED );
#endif

#if DEBUG_SETUPDELAY
	debug(( "setup() - Startup delay: %d ms\n", DEBUG_SETUPDELAY ));
	delay( DEBUG_SETUPDELAY ); /* laptop console delay, prevents missing the intitial serial data */
#endif

	rlrd.rlrd_GlobalMode              = RLRD_GLOBALMODE_AUTO; /* this after all is a clock */
	rlrd.rlrd_Flags                   = 0;
	rlrd.rlrd_GlobalDisplayBrightness = 0;
#if ENABLE_TESTMODE
	rlrd.rlrd_TestMode                = RLRD_TESTMODE_OFF; /* we do not want to kill ourself at startup */
#endif

/* prepare to load and manage settings */

	Global_Setup();   /* setup SPIFFS and some defaults */

/* we reset settings to defaults, so a loaded smaller settings structure will get properly initialized data at the end */

	Global_SettingsReset();  /* if old settings are smaller than store ones, we at least init the new values */
	if( Global_SettingsLoad() ) {
		Global_SettingsReset();  /* loading failed, so we use our defaults again */
	}

/* Setup the loop task on the second core */

	xTaskCreatePinnedToCore(
		CodeLoopCore0,              // task function. 
		"core0Loop",                // name of task. 
		10000,                      // stack size of task 
		NULL,                       // parameter of the task 
		0,                          // priority of the task 
		&core0Loop,                 // task handle to keep track of created task 
		0                           // pin task to core 0
	);

/* get time data if possible */
	Time_Startup();

/* prepare sound module if available */
	Sound_Setup();

/* prepare render engine */
	Render_Setup();

/* now init the test mode, if compiled in */
#if ENABLE_TESTMODE
	TestMode_Setup();
#endif

/* setup animations and effects */
	Animation_Setup();

/* setup led handler */
	Display_Startup();

/*
** This is basically a fancy delay loop with an external method to lenghen the wait. 
**
** OTA is actually handled by the second core, which we use for OTA, wifi and
** the webserver processing.
**
** Here we simply wait OTA_STARTLOOPDELAY times one milli second. If the OTA update state
** gets set, we convert the delay into an endless loop, as we want to finish the update
** performed by the second core and not potential triggering random reboot by crashing 
** the first core.
**
** By default the delay is turned off. If you want to experiment with the code and dont 
** have the device connected via USB, set the OTA_STARTLOOPDELAY and you have a back 
** door, when your code crashes after boot up.
**
** 5000 gives you a more or less five second time window to start the update process over 
** the air. Slower or bigger values are possible. 5 seconds is enough, as the update server
** is frequently sending requests and ota_updatestate flag prevents bad things from happening.
**
*/

#if ENABLE_OTA           /* is ota required to be compiled in? */
#if OTA_STARTLOOPDELAY   /* delay is also acting as code removed to save space */
	{
		int32_t i = OTA_STARTLOOPDELAY;

		debug(( "setup() - Waiting for potential OTA launch attempt\nEstimated delay: ~%d ms\n", OTA_STARTLOOPDELAY ));

		while( i ) {
			yield();
			delay( 1 ); /* laptop console delay, prevents missing the intitial serial data */
			yield();
			if( !ota_updatestate ) { /* if ota is in progress, we stall the startup */
				i--;
			}
		}
		debug(( "setup() - No OTA attempt detected on startup\n" ));
	}
#endif
#endif
}
/* \\\ */

/* /// loop() on core 1
** 
*/

/***************************************************************************/

void loop()
{
	Time_Handle();
	Global_HandleEvents( false ); /* hour and half hour gong, night time for brightness, sound and power mode */

#if ENABLE_OTA
	if( ota_updatestate ) {

		Display_Clear();
		while( ota_updatestate ) { /* while OTA is running, we halt any processing */
			yield();
			FastLED.delay( 1000 );
			yield();
		}
		FastLED.setBrightness( 0 );
		Display_Clear();
	}
#endif

	switch( rlrd.rlrd_GlobalMode ) {
		case RLRD_GLOBALMODE_AUTO:
			/* switches between off and clock mode */
			if( !( rlrd.rlrd_Flags & RLRDF_STATUS_POWERSTATENIGHT ) ) {
				goto Label_GeneralLoop;
			} 
			//break; /* fall through */
		case RLRD_GLOBALMODE_ALWAYSOFF:
/* We only do this once. Chances are next to zero the non visible area at the bottom right was set to this invalid number.
** In clock mode the entire buffer gets cleared at any frame, too. So this little one way switch will never fail us.
*/
			if( renderbuffer[ RINGLIGHT_SEGMENT_LEDS ] != (CRGB) RENDER_LED_INVALID ) {
				Render_ClearBuffer();
				Display_Update();
				renderbuffer[ RINGLIGHT_SEGMENT_LEDS ] = (CRGB) RENDER_LED_INVALID; /* write our magic disable color */
			}
			break;

#if ENABLE_TESTMODE
		case RLRD_GLOBALMODE_TESTMODE:
			TestMode_Handle();
			Display_Handle();   /* test mode does not use render buffer, so no Display_Update() here */
			break;
#endif
		case RLRD_GLOBALMODE_ALWAYSON:
Label_GeneralLoop:
/* animate everything */
			Animation_Animate();
			Display_Update();
			break;
	} /* switch */

	yield();
	FastLED.delay( 1000 / FRAMES_PER_SECOND );
	yield();
}
/* \\\ */


/* ///  loop() on core 0
**
** We use this for server stuff to not interrupt the clock and animation
** core 
*/

/***************************************************************************/

static void CodeLoopCore0( void *)
{

/* first setup station mode, or - if not possible - access point mode */

	Wifi_SetupStationMode();   /* this inits wifi state engine */

/* webserver startup requires wifi to be setup before */

	WebServer_Setup();

#if ENABLE_OTA
	OTA_Startup();
#endif

	for(;;) { 
		Wifi_Handle();
		ArduinoOTA.handle();
		yield();
		delay(1);
	} /* for */
	
}
/* \\\ */
