#ifndef RL_MAIN_H
#define RL_MAIN_H 1

/***************************************************************************/

#define VERSION                          1
#define REVISION                        17

#define DAY                             24
#define MONTH                            4
#define YEAR                          2021

#define APPLICATIONNAME                  "Ring Light"
#define APPLICATIONNAME_NS               "RINGLIGHT"
#define AUTHORNAME                       "Guido Mersmann"
#define WEBSITETITLE                     "Geit@Home - Ring Light Server"
#define WEBSITEFOOTER                    "Ring Light Server"

#define WIFI_CREDENTIALSFILE             1  /* use "//include/credentials.h" */


/***************************************************************************/

/*
** macros for value to string
*/

#define I2S(x) I2S2(x)
#define I2S2(x) #x

/***************************************************************************/

/*
** debug related 
*/

#define ENABLE_DEBUG                     1  /* turn on debug lines */
#define ENABLE_DEBUGEXTENDED             0  /* turn on additional debug lines */
#define ENABLE_TESTMODE                  0  /* turn on test patterns and animations */

#define DEBUG_SETUPDELAY                 0  /* wait 5 seconds before starting setup code */
#define DEBUG_SERIALSPEED           115200  /* serial data transfer speed */

/***************************************************************************/

/* more flexible and handy debug functionality */

#if ENABLE_DEBUG
  #define debug(x) Serial.printf x
  #if ENABLE_DEBUGEXTENDED
    #define debugex(x) Serial.printf x
  #else
    #define debugex(x)
  #endif
#else
  #define debug(x)
  #define debugex(x)
#endif

/***************************************************************************/

/*
** OTA related
*/

#define ENABLE_OTA                       1  /* enable over the air programming */

#define OTA_STARTLOOPDELAY               0  /* wait 60 seconds for OTA, before leaving init code */

/***************************************************************************/

/*
** timing related
*/

#define FRAMES_PER_SECOND              120  /* speed limiter LED core */

#define FASTLED_INTERNAL                 1  /* silence FastLED warning output */
//#define FASTLED_ALLOW_INTERRUPTS         0

/***************************************************************************/

/*
** important defaults
*/

#define PSU_VOLTAGE                      5  /* voltage our PSU provides */
#define PSU_MILLIAMPS                 1500  /* milliamps our PSU provides (fastLED will reduce brightness when needed) */

/***************************************************************************/

/*
** functionality defaults
*/

#define DISPLAY_BRIGHTNESS_NIGHTSTART   21  /* hour */
#define DISPLAY_BRIGHTNESS_NIGHTEND      8  /* hour */
#define DISPLAY_BRIGHTNESS_MIN           0  
#define DISPLAY_BRIGHTNESS_MAX         100  /* clock face settings can only reduce the brightness */
#define DISPLAY_BRIGHTNESS_DAY         ( DISPLAY_BRIGHTNESS_MAX / 2 )
#define DISPLAY_BRIGHTNESS_NIGHT       ( DISPLAY_BRIGHTNESS_DAY / 4 )

#define SOUND_VOLUME_NIGHTSTART          8  /* hour */
#define SOUND_VOLUME_NIGHTEND            8  /* hour */
#define SOUND_VOLUME_MAX               100  
#define SOUND_VOLUME_DAY               ( SOUND_VOLUME_MAX     )
#define SOUND_VOLUME_NIGHT             ( SOUND_VOLUME_DAY / 4 )

#define POWER_STATE_NIGHTSTART           3  /* hour */
#define POWER_STATE_NIGHTEND             8  /* hour */

/***************************************************************************/

/*
** OTA and other header files used
*/

#include <Arduino.h>
#include <FastLED.h>
#include <SPIFFS.h>

#if ENABLE_OTA
#include <ArduinoOTA.h>
#endif

/***************************************************************************/

/*
** RingLightConfig
**
** This is the global configuration structure, which gets saved to flash
*/

#define WIFI_SSID_SIZEOF 32
#define WIFI_PASS_SIZEOF 32

#define WEBSERVER_PASSWORD_SIZEOF 32

struct RingLightConfig {
	uint32_t  rlc_Version;                             /* in case the order gets changed, we need to change the structure version */
	uint32_t  rlc_Flags;                               /* flags as defined below */
	uint32_t  rlc_AnimationMode;                       /* visual style the device is using */
	uint32_t  rlc_AnimationModeRandom;                 /* each bit represents an animation type used in random mode */
	uint32_t  rlc_Brightness;                          /* overall brightness */
	CRGB      rlc_RGBColor;                            /* main color depending on animation mode */
	CRGB      rlc_RGBColorAlternate;                   /* secondary color depending on animation mode */
	uint32_t  rlc_DisplayBrightnessNightStart;         /* hour of day, when the display goes darker */
	uint32_t  rlc_DisplayBrightnessNightEnd;           /* hour of day, when the display goes brigher */
	uint32_t  rlc_DisplayBrightnessNight;              /* 0-100 display brightness during night time */
	uint32_t  rlc_DisplayBrightnessDay;                /* 0-100 display brightness during day time */
	uint32_t  rlc_SoundVolumeNightStart;               /* hour of day, when sound volume goes down */
	uint32_t  rlc_SoundVolumeNightEnd;                 /* hour of day, when sound volume goes up */
	uint32_t  rlc_SoundVolumeNight;                    /* 0-100 sound volume during night time */
	uint32_t  rlc_SoundVolumeDay;                      /* 0-100 sound volume during day time */
	uint32_t  rlc_SoundIDBoot;                         /* startup sound id (power on or reboot only) */
	uint32_t  rlc_SoundIDHour;                         /* sound id for hour gong */
	uint32_t  rlc_SoundIDHalfHour;                     /* sound id for 30 minute gong */
	uint32_t  rlc_PowerStateNightStart;                /* hour of day, when the device shuts off and only does WEBSERVER & OTA */
	uint32_t  rlc_PowerStateNightEnd;                  /* hour of day, when the device turns to normal operation mode again */
	uint32_t  rlc_LEDColorCorrection;                  /* LED type color correction method */
	uint32_t  rlc_LEDColorTemperature;                 /* LED type color temperature method */
	char      rlc_WifiSSID[ WIFI_SSID_SIZEOF + 2 ];    /* Wifi SSID data */
	char      rlc_WifiPass[ WIFI_PASS_SIZEOF + 2 ];    /* Wifi password data */
	char      rlc_WebServerPassword[ WEBSERVER_PASSWORD_SIZEOF + 2 ];    /* Webserver password data */
};

#define RLC_VERSION                      0             /* change if structure above get altered other than adding something to the end */

#define RLCB_SOUNDENABLE                 0             /* sound on/off setting */

#define RLCF_SOUNDENABLE  ( 1 << RLCB_SOUNDENABLE )    /* sound on/off setting */

/***************************************************************************/

/*
** RingLightRuntimeData
**
** This structure stores data apart from settings to detect changes
** or perform other operations.
*/

struct RingLightRuntimeData {
	uint32_t     rlrd_Flags;
	uint32_t     rlrd_GlobalMode;
	uint32_t     rlrd_GlobalDisplayBrightness;             /* used for night mode change detection */
#if ENABLE_TESTMODE
	uint32_t     rlrd_TestMode;
#endif
	uint32_t     rlrd_LEDColorCorrection;
	uint32_t     rlrd_LEDColorTemperature;
};

/***************************************************************************/

/* 
** global mode defines
**
** global modes define the general purpose of the display.
** some are self terminating like count down timers
** will return to clock mode after finishing their job.
*/

enum {
  RLRD_GLOBALMODE_AUTO              = 0,  /* alternate between light and off mode depending on day/night settings */
  RLRD_GLOBALMODE_ALWAYSOFF,              /* show nothing */
  RLRD_GLOBALMODE_ALWAYSON,               /* power depends on power settings */
#if ENABLE_TESTMODE
  RLRD_GLOBALMODE_TESTMODE,               /* test mode, see rl_testmode.cpp */
#endif
};

/* global runtime data flags */

#define RLRDB_STATUS_POWERSTATENIGHT               0   /* current power state (depending on time) */
#define RLRDB_STATUS_SOUNDVOLUMENIGHT              1   /* current sound volume state (depending on time) */
#define RLRDB_STATUS_DISPLAYBRIGHTNESSNIGHT        2   /* current display brightness state (depending on time) */

#define RLRDB_STATUS_TIMEVALID                     3   /* current time data is valid */
#define RLRDB_STATUS_NETWORK                       4   /* we are connected to wifi */

#define RLRDF_STATUS_POWERSTATENIGHT          ( 1 << RLRDB_STATUS_POWERSTATENIGHT )
#define RLRDF_STATUS_SOUNDVOLUMENIGHT         ( 1 << RLRDB_STATUS_SOUNDVOLUMENIGHT )
#define RLRDF_STATUS_DISPLAYBRIGHTNESSNIGHT   ( 1 << RLRDB_STATUS_DISPLAYBRIGHTNESSNIGHT )

#define RLRDF_STATUS_TIMEVALID                ( 1 << RLRDB_STATUS_TIMEVALID )
#define RLRDF_STATUS_NETWORK                  ( 1 << RLRDB_STATUS_NETWORK )

/***************************************************************************/

extern struct RingLightConfig rlc;

extern struct RingLightRuntimeData rlrd;

/***************************************************************************/

/*
**
** this section contains the used pins and other hardware defaults.
**
** add any hardware config related stuff in this section to keep it
** together.
**
*/

#define RINGLIGHT_SEGMENT_LEDTYPE            WS2812B   /* hardware type used */
#define RINGLIGHT_SEGMENT_LEDS                    57   /* total number of LEDs. This depends on the used LED components and scale */
#define RINGLIGHT_SEGMENT_1_PIN                   23   /* data out pin, to be connected to the LED strip */

/***************************************************************************/

#endif /* RL_MAIN_H */
