/*
**   rl_animation.cpp
**
**   Copyright 2021 Guido Mersmann <geit@geit.de>
**   All rights reserved.
**
*/

/***************************************************************************/

#include "rl_animation.h"
#include "rl_render.h"

/***************************************************************************/

struct RL_Animation animation;

/***************************************************************************/

/* /// Animation_Setup() 
**
*/

/***************************************************************************/

void Animation_Setup( void )
{

	debug(( "\nAnimation_Setup()\n" ));

	memset( &animation, 0, sizeof( struct RL_Animation ) );
	animation.Flags = ACF_FADEOUT|ACF_ACTIVE; /* make the animation fade in on start */

	Animation_Randomizer(); /* setup random mode, just in case random mode is used */

}
/* \\\ */

/* /// Animation_Randomizer()
**
*/

/***************************************************************************/

void Animation_Randomizer( void )
{
uint32_t i, rnd;
/* remove random from bit field, as it makes no sense */
	rlc.rlc_AnimationModeRandom &= ~(1<<ANIMATION_RANDOM);

/* no random mode, then reset to defaults */
	if( !rlc.rlc_AnimationModeRandom ) { /* no random modes defined? -> */
		debug(( "Animation_Randomizer() - No random preset -> using defaults\n" ));
		rlc.rlc_AnimationModeRandom = ANIMATION_RANDOM_MODEFLAGS; /* reset to defaults */
	}
/* if current mode is the only mode, then there is no need to change */
	rnd = rlc.rlc_AnimationModeRandom;
	if( !( rnd &= ~( 1 << animation.RandomMode ) ) ) {
		debug(( "Animation_Randomizer() - Current mode (%d) is only mode, so no change\n", animation.RandomMode ));
		return; /* only this mode is available, so no change */
	}
/* we use rnd, which lacks the current mode, so the same mode cannot happen twice in a row. */
	while( 1 ) {
		i = random( 0, ANIMATION_LAST-1 ); /* get random mode */
		if( i != animation.Mode  && ( rnd & ( 1 << i ) ) && ( i != ANIMATION_RANDOM ) ) {
			break;
		}
	}
	animation.RandomMode = i; /* we change the sub mode, so fade will notice */
	debug(( "Animation_Randomizer() - new mode is (%d)\n", animation.RandomMode ));
}
/* \\\ Animation_Randomizer() */

/* /// Animation_Animate() 
**
*/

/***************************************************************************/

void Animation_Animate( void )
{
uint32_t i, bright;
CRGB colorf;
CRGB colorb;
CRGB colort;

/* the animation change and its fading is state driven */

	switch( animation.Flags & ( ACF_ACTIVE|ACF_MODEMASK ) ) { 
		case ACF_FADEOUT|ACF_ACTIVE:  /* fade in animation */
			animation.FadeBrightness  -= ANIMATION_FADESTEP;
			if( animation.FadeBrightness <= 0 ) {
				animation.FadeBrightness = 0;
				animation.Flags &= ~(ACF_ACTIVE); /* fade out done */
			}
			break;
		case ACF_FADEOUT: /* fade out animation is done */
			animation.Flags           &= ~(ACF_MODEMASK); /* disable all modes */
			animation.Flags           |= ACF_ACTIVE|ACF_FADEDELAY; /* activate fade delay */
			animation.FadeDelay        = ANIMATION_FADEDELAY; /* setup delay */
			break;
		case ACF_FADEDELAY|ACF_ACTIVE:  /* fade delay */
			if( animation.FadeDelay ) {
				animation.FadeDelay--;
			} else {
				animation.Flags       &= ~(ACF_ACTIVE); /* fade delay done */
			}
			break;
		case ACF_FADEDELAY: /* fade delay is done */
			animation.Flags           &= ~(ACF_MODEMASK); /* disable all modes */
			animation.Flags           |= ACF_ACTIVE|ACF_FADEIN; /* enable fade in  */
			if( ( rlc.rlc_AnimationMode != ANIMATION_RANDOM ) ) {
				animation.Mode             = rlc.rlc_AnimationMode; /* set new mode */
			} else {
				animation.Mode             = animation.RandomMode; /* set new mode */
			}
			animation.FadeBrightness   = 0; /* should be obsolete */
			/* reset all animation related data here */

			animation.Phase            = 0;  /* reset animation phase */
			animation.InternalCount1   = 0;
			animation.InternalCount2   = 0;
			break;
		case ACF_FADEIN|ACF_ACTIVE:  /* fade in animation */
			animation.FadeBrightness  += ANIMATION_FADESTEP;
			if( animation.FadeBrightness >= 100 ) {
				animation.FadeBrightness = 100;
				animation.Flags &= ~(ACF_ACTIVE); /* fade in done */
			}
			break;
		case ACF_FADEIN:  /* fade in animation is done */
			animation.Flags           &= ~(ACF_MODEMASK); /* disable all modes */
			animation.Flags           |= ACF_ACTIVE; /* activate plain display mode */
			break;
		default:
		case ACF_ACTIVE: /* we run in normal animation mode */
			if( ( rlc.rlc_AnimationMode != ANIMATION_RANDOM ) ) {
				if( animation.Mode != rlc.rlc_AnimationMode ) {
					animation.Flags |= ACF_ACTIVE|ACF_FADEOUT; /* turn animation on again. This time "out" mode */
					animation.FadeBrightness  = 100; /* should be obsolete */
				} 
			} else {
				if(  animation.Mode != animation.RandomMode ) { /* did the mode change? */
					animation.Flags |= ACF_ACTIVE|ACF_FADEOUT; /* turn animation on again. This time "out" mode */
					animation.FadeBrightness  = 100; /* should be obsolete */
				}
			}
			break;
	}
	/* pre calculate color based on brightness */
	bright = ( 100 * animation.FadeBrightness / 100 );
	colorf = Render_ChangeBrightness( rlc.rlc_RGBColor, bright );
	colorb = Render_ChangeBrightness( rlc.rlc_RGBColorAlternate, bright );

	switch( animation.Mode ) {
/* /// STATIC */
		case ANIMATION_STATIC:
			Render_SetBuffer( colorf );
			break;
/* \\\ STATIC */
/* /// Rainbow */
		case ANIMATION_RAINBOWSPARKLE:
		case ANIMATION_RAINBOW:

			animation.Phase++;
			animation.Phase &= 0xff;

			fill_rainbow( renderbuffer, RINGLIGHT_SEGMENT_LEDS, animation.Phase, 7 );
/* we need to fade all pixel if bright is lower then 100% */
			if( bright != 100 ) {
				for( i = 0 ; i < RINGLIGHT_SEGMENT_LEDS ; i++ ) {
					renderbuffer[ i ] = Render_ChangeBrightness( renderbuffer[ i ], bright );
				}
			}
/* Sparkle mode is just the normal rainbow with random sparkles */
			if( animation.Mode == ANIMATION_RAINBOWSPARKLE ) {
				if( random8(100) < RAINBOW_SPARKLERATE ) {
					renderbuffer[ random16( RINGLIGHT_SEGMENT_LEDS - 1  ) ] = Render_ChangeBrightness( CRGB( 255, 255, 255 ), bright );
				}
			}
			break;
/* \\\ Rainbow */
/* /// Fireplace*/
		case ANIMATION_FIREPLACE:
		case ANIMATION_ICEPLACE:
		case ANIMATION_GRASSPLACE:
			static byte firedata[ FIRE_HEIGHT ];
			uint32_t tmp, tmp2;
			if( animation.Phase ) {
				animation.Phase--;
				break;
			}
			animation.Phase = FIRE_SPEED; /* reset speed */

/* update fire: cool down */
			for( i = 0 ; i < FIRE_HEIGHT ; i++ ) {
				tmp = random( 0, ( ( FIRE_DOWN * 10 ) / FIRE_HEIGHT ) + 2 );
				if( tmp > firedata[ i ] ) {
					firedata[ i ] = 0;
				} else {
					firedata[ i ] -= tmp;
				}
			}
/* update fire: spread along cells */
			for( i = FIRE_HEIGHT - 1 ; i >= 2; i-- ) {
				firedata[ i ] = (firedata[ i - 1 ] + firedata[ i - 2 ] + firedata[ i - 2 ] ) / 3;
			}

/* update fire: ignite at the bottom of the ring */
			if( random( 255 ) < FIRE_UP ) {
				i = random( FIRE_SPARK_RADIUS );
				firedata[ i ] += random( FIRE_SPARK_MININTENSITY, 255 );
			}

/* render our fire place */

			for( i = 0 ; i < FIRE_HEIGHT ; i++ ) { /* again only fire height and not the entire strip */

				tmp = round( ( firedata[ i ] / 255.0 ) * ( 0xC0 - 1 ) ); /* scale fire data below 0xc0 as we only use three 0x40 sized segments */
				tmp2   = ( tmp & ( 0x40-1 ) ) << 2; /* get lower segment values and upscale to maximum brightnes */

/* we now write fire particles mirrored from start and end of the strip. This way the fire burns up the ring. */
				switch( animation.Mode ) {
				case ANIMATION_FIREPLACE:
					if( tmp > 0x80 ) {
						colorf = Render_ChangeBrightness( CRGB(  255, 255, tmp2 ), bright );   /* hell fire */
						renderbuffer[                          i ] = colorf;
						renderbuffer[ RINGLIGHT_SEGMENT_LEDS - i ] = colorf;
					} else {
						if( tmp > 0x40 ) {
						colorf = Render_ChangeBrightness( CRGB(  255, tmp2, 0 ), bright );   /* medium rare */
							renderbuffer[                          i ] = colorf;
							renderbuffer[ RINGLIGHT_SEGMENT_LEDS - i ] = colorf;
						} else {
						colorf = Render_ChangeBrightness( CRGB(  tmp2, 0, 0 ), bright );   /* frosty */
							renderbuffer[                          i ] = colorf;
							renderbuffer[ RINGLIGHT_SEGMENT_LEDS - i ] = colorf;
						}
					}
					break;
				case ANIMATION_ICEPLACE:
					if( tmp > 0x80 ) {
						colorf = Render_ChangeBrightness( CRGB(  128, 128, tmp2 ), bright );   /* hell fire */
						renderbuffer[                          i ] = colorf;
						renderbuffer[ RINGLIGHT_SEGMENT_LEDS - i ] = colorf;
					} else {
						if( tmp > 0x40 ) {
							colorf = Render_ChangeBrightness( CRGB(  0, tmp2, 255 ), bright );   /* medium rare */
							renderbuffer[                          i ] = colorf;
							renderbuffer[ RINGLIGHT_SEGMENT_LEDS - i ] = colorf;
						} else {
							colorf = Render_ChangeBrightness( CRGB(  0, 0, tmp2 ), bright );   /* frosty */
							renderbuffer[                          i ] = colorf;
							renderbuffer[ RINGLIGHT_SEGMENT_LEDS - i ] = colorf;
						}
					}
					break;
				case ANIMATION_GRASSPLACE:
					if( tmp > 0x80 ) {
						colorf = Render_ChangeBrightness( CRGB(  128, tmp2, 128 ), bright );      /* full */
						renderbuffer[                          i ] = colorf;
						renderbuffer[ RINGLIGHT_SEGMENT_LEDS - i ] = colorf;
					} else {
						if( tmp > 0x40 ) {
							colorf = Render_ChangeBrightness( CRGB(  tmp2, 255, 0 ), bright );   /* medium */
							renderbuffer[                          i ] = colorf;
							renderbuffer[ RINGLIGHT_SEGMENT_LEDS - i ] = colorf;
						} else {
							colorf = Render_ChangeBrightness( CRGB(  0, tmp2, 0 ), bright );   /* frosty */
							renderbuffer[                          i ] = colorf;
							renderbuffer[ RINGLIGHT_SEGMENT_LEDS - i ] = colorf;
						}
					}
					break;
				}
			}
			break;
/* \\\ Fireplace */
/* /// RGBFade */
		case ANIMATION_RGBFADE:
			{ uint8_t gun;
				gun = animation.Phase & 0xff; /* animation.Phase is our color gun value */
				if( animation.Phase >= 256 ) { /* if phase is bigger than 255, then we reverse fading */
					gun = 255 - gun;
				}
				for( i = 0 ; i < RINGLIGHT_SEGMENT_LEDS ; i++ ) {
					switch( ( animation.InternalCount1 % 3 ) ) {
						case 0:
							renderbuffer[ i ] = Render_ChangeBrightness( CRGB( gun, 0, 0 ), bright );
							break;
						case 1:
							renderbuffer[ i ] = Render_ChangeBrightness( CRGB( 0, gun, 0 ), bright );
							break;
						case 2:
							renderbuffer[ i ] = Render_ChangeBrightness( CRGB( 0, 0, gun ), bright );
							break;
					}
				}
				animation.Phase++;
				if( ( animation.Phase == 512 ) ) { /* two cycles, so change color */
					animation.InternalCount1++; /* select next r, g or gun */
					animation.Phase = 0;
				}
			}
			break;
/* \\\ RGBFade */
/* /// TWINKLE */
		case ANIMATION_TWINKLE:
			fadeToBlackBy( renderbuffer, RINGLIGHT_SEGMENT_LEDS, TWINKLE_FADEAWAY );
			renderbuffer[ random( RINGLIGHT_SEGMENT_LEDS ) % RINGLIGHT_SEGMENT_LEDS ] = Render_ChangeBrightness( CRGB( random( 255 ), random( 255 ), random( 255 ) ), bright );
			break;
/* \\\ TWINKLE */
/* /// RACE */
		case ANIMATION_RACE:
			{
			uint32_t hue = 0;
				/* fade previous set pixels towards black to create a trail */
				fadeToBlackBy( renderbuffer, RINGLIGHT_SEGMENT_LEDS, RACE_TRAIL_FADEAWAY );
				for( i = 0 ; i < RACE_NUMBEROF ; i++ ) {
					/* now we use a beatsine with different bpm for each lifeform to randomly move the head pixel */
					renderbuffer[ beatsin16( i + 7, 0, RINGLIGHT_SEGMENT_LEDS - 1 ) ] |= Render_ChangeBrightness( CHSV( hue, 200, 255 ), bright );
					hue += 256 / RACE_NUMBEROF; /* spread color equaly on the color palette depending on the amount of racers */
				}
			}
			break;
/* \\\ RACE */
/* /// COLORCOMET */
		case ANIMATION_COLORCOMETSTATIC:
			fadeToBlackBy( renderbuffer, RINGLIGHT_SEGMENT_LEDS, COLORCOMET_TRAIL_FADEAWAY );
			renderbuffer[ beatsin16( COLORCOMET_BPM, 0, RINGLIGHT_SEGMENT_LEDS - 1 ) ] = colorf;
			break;

		case ANIMATION_COLORCOMET:
			fadeToBlackBy( renderbuffer, RINGLIGHT_SEGMENT_LEDS, COLORCOMET_TRAIL_FADEAWAY );
			renderbuffer[ beatsin16( COLORCOMET_BPM, 0, RINGLIGHT_SEGMENT_LEDS - 1 ) ] = Render_ChangeBrightness( CHSV( animation.InternalCount1++, 255, 192), bright );
			break;
/* \\\ COLORCOMET*/
	} /* switch */
}
/* \\\ */
