/*
**   rl_render.cpp
**
**   Copyright 2021 Guido Mersmann <geit@geit.de>
**   All rights reserved.
**
*/

/***************************************************************************/

#include "rl_render.h"
#include "rl_display.h"

/***************************************************************************/

CRGB *renderbuffer; 

/***************************************************************************/

/* /// Render_Setup()
**
*/

/***************************************************************************/

void Render_Setup( void )
{
	debug(( "\nRender_Setup()\n" ));
	debug(( "Render_Setup() - Allocating %d bytes for render buffer\n", sizeof( struct CRGB ) * RINGLIGHT_SEGMENT_LEDS ));

	renderbuffer = (CRGB*) calloc( sizeof( struct CRGB), RINGLIGHT_SEGMENT_LEDS );
	assert( renderbuffer );

	debug(( "Render_Setup() - Success\n" ));
}
/* \\\ Render_Setup() */


/* /// RGBToEHSV() 
**
** This function translates RGB 8 bit values to 
** float EHSV. This allows lossless manipulation of
** brightness and stuff. :D
**
*/

/***************************************************************************/

static void RGBToEHSV( CRGB color, struct EHSV *ehsv )
{
float r,g,b;
float minc, maxc, diff;

	r = ((float) color.r ) / 255.0;
	g = ((float) color.g ) / 255.0;
	b = ((float) color.b ) / 255.0;

//	minc = min( min( r, g ), b );
//	maxc = max( max( r, g ), b );

	minc = r < g ? (r < b ? r : b) : (g < b ? g : b);
	maxc = r > g ? (r > b ? r : b) : (g > b ? g : b);

	diff = maxc - minc;

	ehsv->H = 0;
	if( minc != maxc ) {
		//ehsv->H = 0;
	//} else {
		if( maxc == r ) {
			ehsv->H = fmod(( 60 * (( g - b ) / diff ) + 360 ), 360.0 );
		} else {
			if( maxc == g ) {
				ehsv->H = fmod(( 60 * (( b - r ) / diff ) + 120 ), 360.0 );
			} else {
				if( maxc == b ) {
					ehsv->H = fmod(( 60 * (( r - g ) / diff ) + 240 ), 360.0 );
				}
			}
		}
	}
	if( maxc == 0 ) {
		ehsv->S = 0;
	} else {
		ehsv->S = ( diff / maxc ) * 100;
	}

	ehsv->V = maxc * 100;

}
/* \\\ */
/* /// EHSVToRGB()
**
*/

/***************************************************************************/

static CRGB EHSVToRGB( struct EHSV *ehsv )
{
CRGB color = 0;
float s = ehsv->S / 100;
float v = ehsv->V / 100;
float C = s * v;
float X = C * ( 1 - fabs( fmod( ehsv->H / 60.0, 2 ) - 1 ) );
float m = v - C;
float r, g, b;

	if( ehsv->H > 360 || ehsv->H < 0 || ehsv->S > 100 || ehsv->S < 0 || ehsv->V > 100 || ehsv->V < 0 ) {

		debug(( "EHSVToRGB() - Error while converting : EHSV invalid\n" ));

		return(color);
	}
	if( ehsv->H >= 0 && ehsv->H < 60 ) {
		r = C, g = X, b = 0;
	}
	else if( ehsv->H >=  60 && ehsv->H < 120 ) {
		r = X, g = C, b = 0;
	}
	else if( ehsv->H >= 120 && ehsv->H < 180 ) {
		r = 0, g = C, b = X;
	}
	else if( ehsv->H >= 180 && ehsv->H < 240 ) {
		r = 0, g = X, b = C;
	}
	else if( ehsv->H >= 240 && ehsv->H < 300 ) {
		r = X, g = 0, b = C;
	}
	else {
		r = C, g = 0, b = X;
	}

	color.r = ( r + m ) * 255;
	color.g = ( g + m ) * 255;
	color.b = ( b + m ) * 255;

	return( color );
}
/* \\\ */

/* /// Render_ChangeBrightness()
**
*/

/***************************************************************************/

CRGB Render_ChangeBrightness( CRGB color, uint32_t percent )
{
struct EHSV ehsv;
CRGB result;

	RGBToEHSV( color, &ehsv );

	ehsv.V = ehsv.V * percent / 100; /* change brightness */

	if( ehsv.V > 100.0 ) { /* check brightness limit */
		ehsv.V = 100.0; /* reset to max if needed */
	}
	result = EHSVToRGB( &ehsv );

	return( result );
}
/* \\\ Render_ChangeBrightness() */


/* /// Render_FadeToColor()
**
** Fade current CRGB to destination CRGB color using linear blending in RGB 
** color space.
**
*/

/***************************************************************************/

/* /// _Fade8BitDigit()
**
** Fade the current 8 bit color digit to the destination using linear blending by 
** a given difference.
*/

static void _Fade8BitDigit( uint8_t& current, const uint8_t destination, uint8_t difference )
{
/* speed up things when color digits are equal */
	if( current == destination ) {
		return;
	}

	if( current < destination ) {
		uint8_t delta = destination - current;
		delta    = scale8_video( delta, difference );
		current += delta;
	} else {
		uint8_t delta = current - destination;
		delta    = scale8_video( delta, difference );
		current -= delta;
	}
}
/* \\\ */

/***************************************************************************/

CRGB Render_FadeToColor( CRGB *current, const CRGB *destination, uint8_t difference )
{
	_Fade8BitDigit( current->red,   destination->red,   difference );
	_Fade8BitDigit( current->green, destination->green, difference );
	_Fade8BitDigit( current->blue,  destination->blue,  difference );

	return( *current );
}
/* \\\ */

/* /// Render_ClearBuffer()
**
*/

/***************************************************************************/

void Render_ClearBuffer( void )
{
		Render_SetBuffer( CRGB::Black );
}
/* \\\ */
/* /// Render_SetBuffer()
**
*/

/***************************************************************************/

void Render_SetBuffer( CRGB color )
{
uint32_t i;

	for( i = 0 ; i < RINGLIGHT_SEGMENT_LEDS ; i++ ) {
		renderbuffer[ i ] = color;
	}
}
/* \\\ */