/*
**   rl_time.cpp
**
**   Copyright 2021 Guido Mersmann <geit@geit.de>
**   All rights reserved.
**
*/

/***************************************************************************/

#include "main.h"
#include "rl_global.h"
#include "rl_time.h"
#include "rl_wifi.h"

/***************************************************************************/

const char* NTP_SERVER       = TIME_NTP_SERVER;
const char* TIMEZONE_INFO    = TIME_NTP_TIMEZONE;

/***************************************************************************/

/*
** runtime data 
*/


struct tm  timecurrent;             /* last time info we got from system */
struct tm  timereference;           /* time change reference */
struct tm *timeinfo;

/* 
** these tables are valid within a second and used to animate
*/

byte digitcur[ DIGITS_NUMBEROF ];   /* one for each segment. "12 23 45" */
byte digitref[ DIGITS_NUMBEROF ];   /* one for each segment */

byte digitflags; /* flags represent a digit change to optimize updates, if needed */


/***************************************************************************/

/* /// Time_Startup()
** 
*/

/***************************************************************************/

void Time_Startup( void )
{
}
/* \\\ Time_Startup() */

/* /// Time_UpdateDigits() 
**
*/

/***************************************************************************/

static void Time_UpdateDigits( void )
{
	digitflags = 0;

	/* time not valid, so just keep silent on the digits */

	if( !( rlrd.rlrd_Flags & RLRDF_STATUS_TIMEVALID ) ) {
		return;
	}

/* handle hours left digit */

	digitref[ DIGITB_HOURS_HIGH ] = timereference.tm_hour / 10;
	digitcur[ DIGITB_HOURS_HIGH ] = timecurrent.tm_hour   / 10;

	if( ( digitref[ DIGITB_HOURS_HIGH ] != digitcur[ DIGITB_HOURS_HIGH ] ) ) {
		digitflags |= DIGITF_HOURS_HIGH;
	}


/* handle hours right digit */

	digitref[ DIGITB_HOURS_LOW ] = timereference.tm_hour % 10;
	digitcur[ DIGITB_HOURS_LOW ] = timecurrent.tm_hour   % 10;

	if( ( digitref[ DIGITB_HOURS_LOW ] != digitcur[ DIGITB_HOURS_LOW ] ) ) {
		digitflags |= DIGITF_HOURS_LOW;
	}

/* handle minutes left digit */

	digitref[ DIGITB_MINUTES_HIGH ] = timereference.tm_min / 10;
	digitcur[ DIGITB_MINUTES_HIGH ] = timecurrent.tm_min   / 10;

	if( ( digitref[ DIGITB_MINUTES_HIGH ] != digitcur[ DIGITB_MINUTES_HIGH ] ) ) {
		digitflags |= DIGITF_MINUTES_HIGH;
	}


/* handle minutes right digit */

	digitref[ DIGITB_MINUTES_LOW ] = timereference.tm_min % 10;
	digitcur[ DIGITB_MINUTES_LOW ] = timecurrent.tm_min   % 10;

	if( ( digitref[ DIGITB_MINUTES_LOW ] != digitcur[ DIGITB_MINUTES_LOW ] ) ) {
		digitflags |= DIGITF_MINUTES_LOW;
	}

/* handle seconds left digit */

	digitref[ DIGITB_SECONDS_HIGH ] = timereference.tm_sec / 10;
	digitcur[ DIGITB_SECONDS_HIGH ] = timecurrent.tm_sec   / 10;

	if( ( digitref[ DIGITB_SECONDS_HIGH ] != digitcur[ DIGITB_SECONDS_HIGH ] ) ) {
		digitflags |= DIGITF_SECONDS_HIGH;
	}


/* handle seconds right digit */

	digitref[ DIGITB_SECONDS_LOW ] = timereference.tm_sec % 10;
	digitcur[ DIGITB_SECONDS_LOW ] = timecurrent.tm_sec   % 10;

	if( ( digitref[ DIGITB_SECONDS_LOW ] != digitcur[ DIGITB_SECONDS_LOW ] ) ) {
		digitflags |= DIGITF_SECONDS_LOW;
	}

/* if at least on digit got changed, turn the changed switch on */

	if( digitflags ) {
		digitflags |= DIGITF_CHANGED; 
	}

}
/* \\\ */
/* /// Time_Handle()
** 
*/

/***************************************************************************/

void Time_Handle( void )
{
time_t now;

	if( !( rlrd.rlrd_Flags & RLRDF_STATUS_TIMEVALID ) ) {
		if( ( rlrd.rlrd_Flags & RLRDF_STATUS_NETWORK ) ) {

			debugex(( "\nTime_Handle() - Network is up, so We check for a valid time\n" ));

			if( time( nullptr ) )  {

				configTime(0,0, NTP_SERVER, "time.nist.gov" );
				configTzTime(TIMEZONE_INFO, NTP_SERVER);

				delay(1000);

				now = time( nullptr );        /* obtain time, once again. hopefully the proper time now */
				timeinfo = localtime( &now );   /* convert to locale time */

				debug(( ctime( &now ) ));  /* dump some debug, so we see the full date time in action */

/* init the current time variable so we are good to go. no need to fill the reference as it
** will be copied over by the regular Time_Handle() call
*/
				memcpy( &timecurrent, timeinfo, sizeof( struct tm ) );

/* Now we change the hour value to an impossible time. This simulates a hour change at 
** startup. In the result brightness and other stuff is in sync without additional work.
**
** This will create an startup audio check as the hour gong will be played
** unless it is night time.
*/

				timecurrent.tm_hour = 99;

				rlrd.rlrd_Flags |= RLRDF_STATUS_TIMEVALID; /* activate all time functions */

			} else {
				debug(( "\nTime_Handle() - Network is up, but no valid time yet\n" ));
			}
		} else {
			debugex(( "\nTime_Handle() - Waiting for network\n" ));
		}
	} else {

/* current time will be our reference time */

		memcpy( &timereference, &timecurrent, sizeof( struct tm ) );

/* obtain time */

		time( &now );

/* convert to locale time */
		timeinfo = localtime(&now);

/* update our current time */

		memcpy( &timecurrent, timeinfo, sizeof( struct tm ) );

/* reset digit changed, as it is only valid for one cycle */

		digitflags &= ~DIGITF_CHANGED;

/* if date/time is different, we need to update the display
** usually it would be enough to just check the seconds, but
** we may have an delay and there is also the startup
** situation. since the order is seconds, minutes, hours we
** will now waste many cycles here anyway.
*/

		if( ( timecurrent.tm_sec  != timereference.tm_sec ) ||
			( timecurrent.tm_min  != timereference.tm_min ) ||
			( timecurrent.tm_hour != timereference.tm_hour ) ) {

			Time_UpdateDigits();   /* make sure we copy and create the digits for easy update */

		}
	}
}
/* \\\ */
