/*
**   rl_wifi.cpp
**
**   Copyright 2021 Guido Mersmann <geit@geit.de>
**   All rights reserved.
**
*/

/***************************************************************************/

#include <WiFi.h>

#include "main.h"
#include "rl_global.h"
#include "rl_html.h"
#include "rl_render.h"
#include "rl_sound.h"
#include "rl_testmode.h"
#include "rl_webserver.h"
#include "rl_wifi.h"

/***************************************************************************/

#define WIFI_LOOP_TIMEBASE 240                /* this is roughly a second */
#define WIFI_MODE_TIMEOUT_DISCONNECT          (    10 * WIFI_LOOP_TIMEBASE )  /* loop time until accepting a disconnect */
#define WIFI_MODE_TIMEOUT_CONNECT             (     5 * WIFI_LOOP_TIMEBASE )  /* loop time until we consider connecting failed */
#define WIFI_MODE_TIMEOUT_APMODE              (  5*60 * WIFI_LOOP_TIMEBASE )  /* loop time until the AP modes goes full cycle into reconnect */

/***************************************************************************/

uint32_t wifi_mode;                                           /* state engine control */
uint32_t wifi_mode_timeout   = WIFI_MODE_TIMEOUT_CONNECT;     /* delay before switching to/from AP mode */

/***************************************************************************/

/* /// Wifi_SetupStationMode()
**
*/

/***************************************************************************/

void Wifi_SetupStationMode( void )
{

	rlrd.rlrd_Flags &= ~RLRDF_STATUS_NETWORK;     /* let device know, that we are offline */

	if( rlc.rlc_WifiSSID[0] && rlc.rlc_WifiPass[0] ) {
		debug(( "\nWifi_SetupStationMode() - credentials available, switching to station mode\n" ));

		WiFi.disconnect();

		WiFi.mode( WIFI_STA );
		WiFi.begin( rlc.rlc_WifiSSID, rlc.rlc_WifiPass );

		wifi_mode         = WIFIMODE_STATE_CONNECTING;     /* we try to connect now */
		wifi_mode_timeout = WIFI_MODE_TIMEOUT_CONNECT;     /* decrease the wifi timeout */
	} else {
		debug(( "Wifi_SetupStationMode() - No credentials, switching to access point mode\n" ));
		Wifi_SetupAccessPointMode( );               /* setup AP mode */
	}
}
/* \\\ Wifi_SetupStationMode() */
/* /// Wifi_SetupAccessPointMode()
**
*/

/***************************************************************************/

void Wifi_SetupAccessPointMode( void )
{
	debugex(( "\nWifi_SetupAccessPointMode() - Switching to access point mode\n" ));

	rlrd.rlrd_Flags &= ~RLRDF_STATUS_NETWORK;     /* let device know, that we are offline */

	WiFi.disconnect();

	WiFi.softAPConfig( IPAddress(192,168,0,1), IPAddress(192,168,0,1), IPAddress(255,255,255,0) );

	WiFi.softAP( WIFI_ACCESSPOINT_NAME );

	debug(( "Wifi_SetupAccessPointMode() - %s\n", WiFi.softAPIP().toString().c_str() ));

	wifi_mode          = WIFIMODE_STATE_APMODE;     /* we turn APMODE on */
	wifi_mode_timeout  = WIFI_MODE_TIMEOUT_APMODE;  /* decrease the wifi timeout */

}
/* \\\ Wifi_SetupAccessPointMode() */
/* /// Wifi_Handle() 
**
** This is basically a normal handle code, but we frequently check the
** connection and switch back to AP mode, if network is gone for some
** time, so the user can swap APs or location and re-setup without the
** need of a button or other control
*/

/***************************************************************************/

void Wifi_Handle( void )
{
	debugex(( "\nWifi_Handle()\n" ));

	wifi_mode_timeout--;   /* decrease the wifi timeout per second */

	switch( wifi_mode ) {
/***************************************************************************/
/* Station mode was initiated, so we wait for connect or timeout */
		case WIFIMODE_STATE_CONNECTING:
			if( WiFi.waitForConnectResult() == WL_CONNECTED ) {
				debug(( "IP: %s\n",  WiFi.localIP().toString().c_str() ));
				wifi_mode         = WIFIMODE_STATE_CONNECTED;          /* we are connected */
				wifi_mode_timeout = WIFI_MODE_TIMEOUT_DISCONNECT;      /* reset wifi timeout */
			} else {
				if( !wifi_mode_timeout ) {
					debugex(( "Wifi_Handle() - WIFIMODE_STATE_CONNECTING: Connection failed. Entering AP mode\n" ));
					Wifi_SetupAccessPointMode();
				} else {
#if ENABLE_DEBUG /* dump current timout state */
					if( !( wifi_mode_timeout % WIFI_LOOP_TIMEBASE ) ) { /* limit amount of debug output */
						debug(( "Wifi_Handle() - WIFIMODE_STATE_CONNECTING: Waiting for connect %ds\n", wifi_mode_timeout/WIFI_LOOP_TIMEBASE ));
					}
#endif
				}
			}
			break;
/***************************************************************************/
/* we are connected and handle the web page. If the timeout get reached, we
** activate  WIFIMODE_STATE_DISCONNECTED
*/
		case WIFIMODE_STATE_CONNECTED:
			if( WiFi.status() == WL_CONNECTED ) {
				wifi_mode_timeout      = WIFI_MODE_TIMEOUT_DISCONNECT;    /* make sure time out never reaches zero while connected */
				rlrd.rlrd_Flags       |= RLRDF_STATUS_NETWORK;            /* let device know, that we are (still) online */
				server.handleClient();                                    /* handle website */
				//debugex(( "Wifi_Handle() - WIFIMODE_STATE_CONNECTED: Handle\n" ));
			} else {
#if ENABLE_DEBUG /* dump current timout state */
				if( !( wifi_mode_timeout % WIFI_LOOP_TIMEBASE ) ) { /* limit amount of debug output */
					debug(( "Wifi_Handle() - WIFIMODE_STATE_CONNECTED: reconnection timeout: %ds\n", wifi_mode_timeout/WIFI_LOOP_TIMEBASE ));
				}
#endif
				/* we are not connected, so we wait for a reconnect or consider us deconnected */
				if( !wifi_mode_timeout ) {   /* timed out? -> station mode */
					debugex(( "Wifi_Handle() WIFIMODE_STATE_CONNECTED: Connection lost, retry with old SSID & Password configuration\n" ));
					Wifi_SetupStationMode();
				}
			}
			break;
/***************************************************************************/
/* AccessPoint mode is running. The user can enter SSID and password, but after
** WIFI_MODE_TIMEOUT_APMODE is reached, we try to reconnect with previous given
** credentials.
*/
		case WIFIMODE_STATE_APMODE:
			debugex(( "Wifi_Handle() - WIFIMODE_STATE_APMODE\n" ));
			server.handleClient();   /* handle access point website */

			if( !wifi_mode_timeout ) {   /* timed out? -> restart station mode */
				debug(( "Wifi_Handle() - WIFIMODE_STATE_APMODE: Retry old SSID & Password configuration\n" ));
				Wifi_SetupStationMode();
				break;
			}

#if ENABLE_DEBUG /* dump current timout state */
			if( !( wifi_mode_timeout % WIFI_LOOP_TIMEBASE ) ) { /* limit amount of debug output */
				debug(( "Wifi_Handle() - WIFIMODE_STATE_APMODE: waiting for recheck network count: %ds\n", wifi_mode_timeout/WIFI_LOOP_TIMEBASE ));
			}
#endif
			break;
	}
}
/* \\\ Wifi_Handle() */


/***************************************************************************/
